/* gdm-touch.c -- Graphic Display Manager Touch Region
   Copyright 2003 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@nerim.fr)

This file is part of GEL.

GEL is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GEL is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GEL; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <gdm/display.h>
#include <gdm/display-raw.h>

/*! Touch the graphic display to force a synchronization of the region.

    The graphic region specified by @b x, @b y, @b width, @b height is
    touched to force a synchronization refresh with the LCD display.
    The region is clipped to the size of the display.  Touching a region
    has an effect on the behavior of \b gdm_refresh.  When refreshing
    occurs, only the touched regions are really updated on the LCD
    display.  The drawing operations take care of touching the minimal
    region that affect the drawing, hence optimizing any refresh.

    @param dp  Display Manager
    @param x   X corner
    @param y   Y corner
    @param width  Width of region
    @param height Height of region
    @see gdm_refresh  */
void
gdm_touch (gdm_display* dp, short x, short y,
           unsigned short width, unsigned short height)
{
  gdm_line* l;
  gdm_mask mask;
  unsigned short d;

  /* Check that x .. width is within the display.  */
  d = width;
  if (d == 0)
    return;

  d += x;
  if (d > GDM_WIDTH)
    d = GDM_WIDTH;
  /* Compute mask representing the touched region.  */
  mask = _gdm_end_mask[(d - 1) >> 4];

  if (x >= GDM_WIDTH)
    return;
  if (x < 0)
    x = 0;

  mask &= _gdm_start_mask[x >> 4];

  /* Likewise for y .. height.  */
  d = height;
  if (d == 0)
    return;

  d += y;
  if (d > GDM_HEIGHT)
    d = GDM_HEIGHT;
  if (y >= GDM_HEIGHT)
    return;
  if (y < 0)
    y = 0;

  l = _gdm_get_raw_line (dp, y);

  /* Get how many lines we span.  */
  d = ((d - 1) >> 3) - (y >> 3);
  while (1)
    {
      l->touched |= mask;
      if (d == 0) break;
      d--;
      l++;
    }
}
