/* spi.c -- SPI Controller
   Copyright 2000, 2001 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@worldnet.fr)

This file is part of GEL.

GEL is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GEL is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GEL; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <sys/param.h>
#include <sys/locks.h>
#include <m68hc11/interrupts.h>
#include <sys/ports.h>
#include <sys/spi.h>


#define M6811_DDRD4 (1 << 4)
#define M6811_DDRD3 (1 << 3)

#define SPI_QUEUE_SIZE (4)
#define spi_queue_first &spi_queue[0]
#define spi_queue_last  &spi_queue[SPI_QUEUE_SIZE]

static unsigned char spi_queue[SPI_QUEUE_SIZE];
static unsigned char *spi_queue_wr;
static unsigned char *spi_queue_rd;
static unsigned char spi_running;

/* Returns a pointer to the next character to send on the SPI.  */
static inline unsigned char*
next_spi (unsigned char *p)
{
  p++;
  if (p == spi_queue_last)
    p = spi_queue_first;

  return p;
}

void __attribute__((interrupt))
spi_interrupt (void)
{
  unsigned char scratch;

  /* Need to read the SPSR first.  */
  scratch = _io_ports[M6811_SPSR];
  if (spi_queue_rd != spi_queue_wr)
    {
      _io_ports[M6811_SPDR] = *spi_queue_rd;
      spi_queue_rd = next_spi (spi_queue_rd);
    }
  else
    {
      scratch = _io_ports[M6811_SPDR];
      spi_running = 0;
    }
}

void
spi_putc (unsigned char c)
{
  unsigned short mask;
  
  while (1)
    {
      unsigned char *nw;
      
      mask = lock ();
      if (spi_running == 0)
        {
          unsigned char scratch;
          
          spi_running = 1;

          /* Need to read SPSR first.  */
          scratch = _io_ports[M6811_SPSR];
          _io_ports[M6811_SPDR] = c;
          restore (mask);
          break;
        }
      
      nw = next_spi (spi_queue_wr);
      if (nw == spi_queue_rd)
	{
	  restore (mask);
	  continue;
	}
      *spi_queue_wr = c;
      spi_queue_wr = nw;
      restore (mask);
      break;
    }
}

/**
 * Initialize the spi package.
 *
 */
void
spi_initialize (unsigned char mode)
{
  spi_queue_wr    = spi_queue;
  spi_queue_rd    = spi_queue;
  spi_running     = 0;

  /* Install the SPI interrupt handler.  */
  set_interrupt_handler (SPI_VECTOR, spi_interrupt);

  mode |= M6811_SPIE | M6811_SPE;
  _io_ports[M6811_SPCR] = mode;

  /* Enable SCK and MOSI outputs.  */
  _io_ports[M6811_DDRD] |= M6811_DDRD4 | M6811_DDRD3;
}

